/*
* IntersectionTrigger v1.1.6 
* IntersectionTrigger utilizes the most modern web technology to trigger anything by intersection. Including scroll-based animations.
* https://sunshine-themes.com/?appID=ss_app_1
*
* Copyright 2024, Sunshine. All rights reserved.
* @license: Released under the personal 'no charge' license, Licensees of commercial or business license are granted additional rights. See http://sunshine-themes.com/?appID=ss_app_1&tab=license for details..
* @author: Sherif Magdy, eng.sherifmagdy.w@gmail.com
*
* Released on: May 9, 2024
*/

"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/plugins/guides/guides.ts
var guides_exports = {};
__export(guides_exports, {
  default: () => guides_default
});
module.exports = __toCommonJS(guides_exports);

// src/constants.ts
var guideDefaultConfig = {
  enter: {
    trigger: {
      backgroundColor: "rgb(0, 149, 0)",
      color: "#000",
      text: "Enter"
    },
    root: {
      backgroundColor: "rgb(0, 149, 0)",
      color: "#000",
      text: "Root Enter"
    }
  },
  leave: {
    trigger: {
      backgroundColor: "#ff0000",
      color: "#000",
      text: "Leave"
    },
    root: {
      backgroundColor: "#ff0000",
      color: "#000",
      text: "Root Leave"
    }
  }
};

// src/helpers.ts
var is = {
  function: (a) => typeof a === "function",
  string: (a) => "string" === typeof a,
  boolean: (a) => "boolean" === typeof a,
  object: (a) => !!a && "object" === typeof a && a !== null && !(a instanceof Array),
  inObject: (obj, prop) => Object.prototype.hasOwnProperty.call(obj, prop),
  num: (a) => typeof a === "number",
  array: (a) => a instanceof Array,
  element: (a) => a instanceof HTMLElement,
  empty: (a) => Object.keys(a).length === 0,
  doc: (a) => a instanceof Document,
  anime: (a) => is.object(a) && is.inObject(a, "animatables") && !is.inObject(a, "add"),
  tl: (a) => is.object(a) && is.inObject(a, "add") && is.function(a.add),
  animeInstance: (a) => is.anime(a) || is.tl(a),
  pixel: (a) => a.includes("px"),
  percent: (a) => a.includes("%"),
  scrollable: (element, dir) => dir ? "y" === dir ? element.scrollHeight > element.clientHeight : element.scrollWidth > element.clientWidth : element.scrollHeight > element.clientHeight || element.scrollWidth > element.clientWidth
};
var getParents = (element) => {
  const parents = [];
  for (let el = element.parentElement; el && !is.doc(el) && el !== document.documentElement; el = el.parentElement) {
    parents.push(el);
  }
  return parents;
};
var mergeOptions = (defaultOptions, customOptions) => {
  const options = { ...defaultOptions };
  for (const [key, value] of Object.entries(customOptions)) {
    const k = key;
    if (is.object(options[k]) && !is.empty(options[k])) {
      if (!is.object(value))
        continue;
      options[k] = mergeOptions(options[k], value);
    } else {
      options[k] = value;
    }
  }
  return options;
};
var getMinMax = (n1, n2) => [n1, n2].sort((a, b) => a - b);
var setElProps = (el, props) => {
  for (const propName in props) {
    el.style[propName] = props[propName];
  }
};

// src/plugins/guides/guides.ts
var Guides = class {
  _it;
  _utils;
  options;
  _guides;
  isVer;
  rootEl;
  _onResizeHandler;
  static pluginName;
  constructor(it) {
    this._registerIntersectionTrigger(it);
    return this;
  }
  _registerIntersectionTrigger(intersectionTrigger) {
    this._it = intersectionTrigger;
    this._utils = this._it._utils;
  }
  init(options) {
    this.options = is.object(options) ? options : {};
    this._guides = [];
    this.isVer = this._utils.isVertical();
    this.rootEl = this._utils.getRoot();
    this._addResizeListener();
    this.update();
  }
  _addResizeListener() {
    this._onResizeHandler = this.update;
    addEventListener("resize", this._onResizeHandler, false);
  }
  _removeResizeListener() {
    removeEventListener("resize", this._onResizeHandler, false);
  }
  _guideCreation(options, triggerEl) {
    const { enter, position, isHigherValue, text, color, backgroundColor } = options;
    const guide = document.createElement("div"), guideThickness = 2;
    guide.id = triggerEl ? enter ? "trigger-enter" : "trigger-leave" : enter ? "root-enter" : "root-leave";
    setElProps(guide, {
      width: this.isVer ? "100px" : `${guideThickness}px`,
      height: this.isVer ? `${guideThickness}px` : "100px",
      position: "absolute",
      zIndex: "9999",
      backgroundColor,
      [this.isVer ? "top" : "left"]: isHigherValue ? `calc( ${position.toString()} - ${guideThickness}px )` : position.toString()
    });
    const createText = () => {
      const verticalAlignment = {
        dir: this.isVer ? isHigherValue ? "bottom" : "top" : "bottom",
        value: this.isVer ? "5px" : "15px"
      };
      const horizontalAlignment = {
        dir: this.isVer ? "right" : isHigherValue ? "right" : "left",
        value: this.isVer ? triggerEl ? "0px" : !this._it._root ? "15px" : "0px" : "5px"
      };
      const textElement = document.createElement("span");
      textElement.innerText = text;
      guide.appendChild(textElement);
      setElProps(textElement, {
        position: "absolute",
        color,
        fontSize: "16px",
        fontWeight: "bold",
        backgroundColor,
        padding: "5px",
        width: "max-content",
        [verticalAlignment.dir]: verticalAlignment.value,
        [horizontalAlignment.dir]: horizontalAlignment.value
      });
    };
    createText();
    this._guides.push(guide);
    document.body.append(guide);
    if (!triggerEl) {
      setElProps(guide, {
        [this.isVer ? "width" : "height"]: !this._it._root ? "100%" : "100px",
        position: !this._it._root ? "fixed" : "absolute"
      });
      !this._it._root && setElProps(guide, { [this.isVer ? "right" : "top"]: "0px" });
      if (this._it._root)
        this._positionGuide(guide, { enter, isHigherValue });
      return;
    }
    this._positionGuide(guide, { position, enter, isHigherValue }, triggerEl);
    getParents(triggerEl).forEach((parent) => {
      if (!is.scrollable(parent))
        return;
      parent.addEventListener(
        "scroll",
        () => this._positionGuide(guide, { position, enter, isHigherValue }, triggerEl),
        false
      );
    });
  }
  createGuides() {
    const guideParams = mergeOptions(guideDefaultConfig, this.options);
    const guideTextPrefix = this._it.name;
    const rEPValue = this._it._positionsData.rEP.value;
    const rLPValue = this._it._positionsData.rLP.value;
    const rHigherPosition = getMinMax(rEPValue, rLPValue)[1];
    this._guideCreation({
      enter: true,
      position: `${rEPValue}${this._it._positionsData.rEP.unit}`,
      isHigherValue: rHigherPosition === rEPValue,
      text: `${guideTextPrefix} ${guideParams.enter.root.text}`,
      color: guideParams.enter.root.color,
      backgroundColor: guideParams.enter.root.backgroundColor
    });
    this._guideCreation({
      enter: false,
      position: `${rLPValue}${this._it._positionsData.rLP.unit}`,
      isHigherValue: rHigherPosition === rLPValue,
      text: `${guideTextPrefix} ${guideParams.leave.root.text}`,
      color: guideParams.leave.root.color,
      backgroundColor: guideParams.leave.root.backgroundColor
    });
    this._it.triggers.forEach((trigger) => {
      const { enter, leave, higherPosition } = this._utils.getTriggerData(trigger);
      this._guideCreation(
        {
          enter: true,
          position: enter,
          isHigherValue: enter === higherPosition,
          text: `${guideTextPrefix} ${guideParams.enter.trigger.text}`,
          color: guideParams.enter.trigger.color,
          backgroundColor: guideParams.enter.trigger.backgroundColor
        },
        trigger
      );
      this._guideCreation(
        {
          enter: false,
          position: leave,
          isHigherValue: leave === higherPosition,
          text: `${guideTextPrefix} ${guideParams.leave.trigger.text}`,
          color: guideParams.leave.trigger.color,
          backgroundColor: guideParams.leave.trigger.backgroundColor
        },
        trigger
      );
    });
  }
  _positionGuide(guide, options, triggerEl) {
    const { position, isHigherValue } = options;
    const guideBounds = guide.getBoundingClientRect();
    const rDefaultBounds = this.rootEl.getBoundingClientRect();
    let targetBounds = this._utils.getRootRect(this._it.observer.rootMargin);
    let diffs;
    const secAxisRefs = ["right", "left", "bottom", "top"];
    const verMainAxisRef = isHigherValue ? "bottom" : "top";
    const hozMainAxisRef = isHigherValue ? "right" : "left";
    diffs = this.isVer ? {
      x: rDefaultBounds[secAxisRefs[0]] - guideBounds[secAxisRefs[1]],
      y: targetBounds[verMainAxisRef] - guideBounds[verMainAxisRef]
    } : {
      x: targetBounds[hozMainAxisRef] - guideBounds[hozMainAxisRef],
      y: rDefaultBounds[secAxisRefs[2]] - guideBounds[secAxisRefs[3]]
    };
    if (triggerEl) {
      targetBounds = triggerEl.getBoundingClientRect();
      diffs = this.isVer ? {
        x: targetBounds.right - guideBounds.right,
        y: targetBounds.top + position * targetBounds.height - guideBounds.top
      } : {
        x: targetBounds.left + position * targetBounds.width - guideBounds.left,
        y: targetBounds.top - guideBounds.top
      };
    }
    this._setGuideTransformProp(guide, diffs.x, diffs.y);
  }
  _setGuideTransformProp(guide, diffX, diffY) {
    const parts = [...guide.style.transform.matchAll(/(-?\d*\.?\d+)\s*(px|%)?/g)];
    const translateXInPx = parts.length ? parts[0][1] : "0";
    const translateYInPx = parts.length > 1 ? parts[1][1] : "0";
    const x = diffX + parseFloat(translateXInPx);
    const y = diffY + parseFloat(translateYInPx);
    setElProps(guide, { transform: `translate(${x}px,${y}px)` });
  }
  removeGuides = () => {
    this._guides.forEach((guide) => guide && guide.remove());
    this._guides = [];
  };
  update = () => {
    this.removeGuides();
    this.createGuides();
  };
  kill = () => {
    this._removeResizeListener();
    this.removeGuides();
    this._it.guides = this._it = this._utils = void 0;
  };
};
Guides.pluginName = "guides";
var guides_default = Guides;
