/*
* IntersectionTrigger v1.1.6 
* IntersectionTrigger utilizes the most modern web technology to trigger anything by intersection. Including scroll-based animations.
* https://sunshine-themes.com/?appID=ss_app_1
*
* Copyright 2024, Sunshine. All rights reserved.
* @license: Released under the personal 'no charge' license, Licensees of commercial or business license are granted additional rights. See http://sunshine-themes.com/?appID=ss_app_1&tab=license for details..
* @author: Sherif Magdy, eng.sherifmagdy.w@gmail.com
*
* Released on: May 9, 2024
*/


// src/constants.ts
var defaultToggleClassConfig = {
  targets: [],
  toggleActions: "add remove add remove",
  classNames: ""
};

// src/helpers.ts
var is = {
  function: (a) => typeof a === "function",
  string: (a) => "string" === typeof a,
  boolean: (a) => "boolean" === typeof a,
  object: (a) => !!a && "object" === typeof a && a !== null && !(a instanceof Array),
  inObject: (obj, prop) => Object.prototype.hasOwnProperty.call(obj, prop),
  num: (a) => typeof a === "number",
  array: (a) => a instanceof Array,
  element: (a) => a instanceof HTMLElement,
  empty: (a) => Object.keys(a).length === 0,
  doc: (a) => a instanceof Document,
  anime: (a) => is.object(a) && is.inObject(a, "animatables") && !is.inObject(a, "add"),
  tl: (a) => is.object(a) && is.inObject(a, "add") && is.function(a.add),
  animeInstance: (a) => is.anime(a) || is.tl(a),
  pixel: (a) => a.includes("px"),
  percent: (a) => a.includes("%"),
  scrollable: (element, dir) => dir ? "y" === dir ? element.scrollHeight > element.clientHeight : element.scrollWidth > element.clientWidth : element.scrollHeight > element.clientHeight || element.scrollWidth > element.clientWidth
};
var splitStr = (st) => st.trim().split(/\s+/);
var mergeOptions = (defaultOptions, customOptions) => {
  const options = { ...defaultOptions };
  for (const [key, value] of Object.entries(customOptions)) {
    const k = key;
    if (is.object(options[k]) && !is.empty(options[k])) {
      if (!is.object(value))
        continue;
      options[k] = mergeOptions(options[k], value);
    } else {
      options[k] = value;
    }
  }
  return options;
};

// src/plugins/toggleclass/toggleclass.ts
var ToggleClass = class {
  _it;
  _utils;
  static pluginName;
  constructor(it) {
    this._registerIntersectionTrigger(it);
    return this;
  }
  _registerIntersectionTrigger(intersectionTrigger) {
    this._it = intersectionTrigger;
    this._utils = this._it._utils;
  }
  toggle(trigger, toggleClass, eventIndex) {
    for (const { targets, toggleActions, classNames } of toggleClass) {
      const action = toggleActions[eventIndex];
      if ("none" === action)
        continue;
      classNames.forEach((className) => {
        if (!targets.length)
          return trigger.classList[action](className);
        targets.forEach((target) => target.classList[action](className));
      });
    }
  }
  parse(params) {
    let mergedParams = [];
    if (is.string(params))
      mergedParams = [
        mergeOptions(defaultToggleClassConfig, {
          classNames: params
        })
      ];
    if (is.array(params))
      mergedParams = params.map((obj) => mergeOptions(defaultToggleClassConfig, obj));
    return mergedParams.map((obj) => ({
      targets: this._utils.parseQuery(obj.targets, "targets"),
      toggleActions: splitStr(obj.toggleActions),
      classNames: splitStr(obj.classNames)
    }));
  }
  kill = () => {
    this._it = void 0;
    this._utils = void 0;
  };
};
ToggleClass.pluginName = "toggleClass";
var toggleclass_default = ToggleClass;
export {
  toggleclass_default as default
};
